template<class T> // auxiliary class
struct auto_ptr_ref {
  T* ptr;
  auto_ptr_ref (T* p): ptr(p) { }
};

template<class T>
class auto_ptr {
public:
  explicit auto_ptr(T* ptr = 0) throw(): ptr_(ptr) {} 
  auto_ptr(auto_ptr& a) throw(): ptr_(a.release()) {} // non-const parameter!
  auto_ptr(auto_ptr_ref<T> a) throw(): ptr_(a.ptr) {}
  template<class Y>
  auto_ptr(auto_ptr<Y>& a) throw(): ptr_(a.release()) {}
  auto_ptr& operator=(auto_ptr& a) throw() { reset(a.release()); return *this; }
  auto_ptr& operator=(auto_ptr_ref<T> a) throw() { reset(a.ptr); return *this; }
  template<class Y>
  auto_ptr& operator= (auto_ptr<Y>& a) throw() { reset(a.release()); return *this; }
  ~auto_ptr() throw() { delete ptr_; }
  T* get() const throw() { return ptr_; }
  T& operator*() const throw() { return *ptr_; }
  T* operator->() const throw() { return ptr_; }

  T* release() throw() { // release ownership, return ptr_
    T* tmp(ptr_);
    ptr_ = 0;
    return tmp;
    }
  void reset (T* ptr=0) throw() {
    if (ptr_ != ptr) {
      delete ptr_;
      ptr_ = ptr;
      }
    }
  template<class Y> // conversion to auxiliary auto_ptr_ref
  operator auto_ptr_ref<Y>() throw() { return auto_ptr_ref<Y>(release()); }
  template<class Y> // conversion from auto_ptr<Derived*> to auto_ptr<Base*>
  operator auto_ptr<Y>() throw() { return auto_ptr<Y>(release()); }
private:
  T* ptr_; // pointer to the owned object (or 0)
};
