#ifndef	AUTOSTRING_H
#define	AUTOSTRING_H
// $Id$

#include <assert.h>
#include <string.h> // for strcpy, strcat
#include <iostream>
// The auto_ptr implementation in <memory> for g++ 2.95.2 does
// not contain autoptr<T>::operator(auto_ptr_ref<T>) which
// causes a compilation error. Therefore, we use the implementation
// of p. 134 (see also Josuttis ``The C++ standard library''),
// pp. 56 - 57).
#include "autoptr.h"
//#include	<memory> // for auto_ptr

class String { 
// A simple string class, implemented using auto_ptr.
public:
  String(const char* cstring=""): data_(init(cstring?cstring:"")) {}
  String(const String& s): data_(init(s.data())) {} 

  String& operator=(const String& s) {
    if (this==&s) // do nothing if self-assignment
      return *this;
    data_ = init(s.data()); // copy C string from parameter
    return *this;
    }

  // Return const to prevent change by caller.
  const char* data() const { return data_.get(); }

  // Return length of string.
  unsigned int size() const { return (data_.get() ? strlen(data_.get()) : 0); } 

  // Concatenate s to *this.
  String& concat(const String& s) {
    // Remember the size of this string.
    size_t sz(size()); 
    // Transfer ownership of data_ to old and set data_ to 0.
    auto_ptr<char> old(data_); 
    // Allocate buffer large enough to hold both strings + trailing '\0'
    data_ = auto_ptr<char>(new char[sz+s.size()+1]); 
    // Use C library functions to copy the strings to the new data_ area.
    strcat(strcpy(data_.get(),old.get()),s.data()); // use C lib functions
    // The destructor of old will automatically delete the old data_ area.
    return *this; 
    }
private:
  auto_ptr<char> data_; // A C string, i.e. a 0-terminated array of char.

  // Auxiliary function for constructors.
  auto_ptr<char> init(const char* s) {
    assert(s);
    return auto_ptr<char>(strcpy(new char[strlen(s)+1],s));
    }
};

String
operator+(const String& s1,const String& s2) { // concatenation
return String(s1).concat(s2);
}

ostream&
operator<<(ostream& os,const String& s) {
return os << s.data();
}
#endif
