#ifndef	QSORT_H
#define	QSORT_H
// $Id$

// Generic quicksort function where the comparsion operator is a
// template parameter of function type.

template <typename T>
inline void
exchange(T* a,int i,int j) {
// Auxiliary function template.
T tmp(a[i]);
a[i] = a[j];
a[j] = tmp;
}

template <typename T,bool (*less_than)(const T&,const T&)>
void
quicksort(T* a,int low,int high) { 
// Quicksort: sort a[low..high].

if (low<high) { // Area to be sorted has >1 elements.
  if (less_than(a[high],a[low])) 
    // Ensure that a[low]<= pivot, a[high]>= pivot; this ensures that
    // the inner loops do not run out of the range.
    exchange(a,low,high); 
  int lo(low);
  int hi(high+1);
  T pivot(a[low]); 
  // We will split a[low..high] into 3 areas:
  // a[low..hi-1]<= pivot, a[hi]=pivot, and a[hi+1..high]>= pivot.
  for (;;) { 
    // Move lo to the right until a[lo]>=pivot.
    do ++lo; while (less_than(a[lo],pivot)); 
    // Move hi to the left until a[hi]<=pivot.
    do --hi; while (less_than(pivot,a[hi])); 
    if (lo<hi) 
      // A[lo]>=pivot and a[hi] <= pivot, thus a[hi] belongs in left part
      // and a[lo] belongs in right part: swap and continue.
      exchange(a,lo,hi); 
    else 
      // A[hi+1..high] >= pivot, a[low+1..hi] <= pivot and a[hi]<=pivot.
      break; 
    }
  exchange(a,low,hi); 
  // Now a[low..hi-1] <= pivot, a[hi] = pivot,  a[hi+1..high] >=
  // pivot. Sort the first and third areas.
  quicksort<T,less_than>(a,low,hi-1);
  quicksort<T,less_than>(a,hi+1,high);
  }
}
#endif
