#ifndef	STACK_H
#define	STACK_H
// $Id$

// Demo of class template friendship.

#include <assert.h> // It would be better to use exceptions.

template<typename T> class Stack; // Forward declaration.

template<typename Value>
class Node {
// A Node in a singly linked list containing a data item of type Value.
friend class Stack<Value>; // only Stack<Value> can access a Node<Value> object

private: // this is the default, so this line could be dropped
  Node(const Value& v,Node* next=0): item_(v), next_(next) {}
  Node*	 next() const { return next_; }
  const Value& item() const { return item_; }
  Value	 item_; // contents of the node
  Node*	 next_; // pointer to next node or 0.
};

template<typename T>
class Stack { 
// A stack is implemented as a linked list of nodes, starting with the top.
public:
  Stack(): top_(0) {}
  ~Stack() { while (top_) pop(); }

  Stack&    push(const T& t) { 
    	      top_ = new Node<T>(t,top_); 
	      return *this; 
	      }

  const T&  top() const { assert(top_); return top_->item(); }

  // The pop() member function returns the element that was popped.
  T	pop() { 
          assert(top_); 
	  Node<T>* tmp(top_->next()); // Pointer to node under top or 0
	  T value(top());
	  delete top_;
	  top_ = tmp;
	  return value;
	  }
  bool		empty() const { return top_ == 0; }
private:
  Node<T>*	top_; // Pointer to topmost node.
};
#endif
